#!/usr/bin/python
# -*- coding: UTF-8 -*-

############################################################################
##                                                                        ##
## Bookkun - Manage your ebooks.                                          ##
##                                                                        ##
## Copyright (C) 2012  Fabien Loison <flo AT flogisoft DOT com>           ##
##                                                                        ##
## This program is free software: you can redistribute it and/or modify   ##
## it under the terms of the GNU General Public License as published by   ##
## the Free Software Foundation, either version 3 of the License, or      ##
## (at your option) any later version.                                    ##
##                                                                        ##
## This program is distributed in the hope that it will be useful,        ##
## but WITHOUT ANY WARRANTY; without even the implied warranty of         ##
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          ##
## GNU General Public License for more details.                           ##
##                                                                        ##
## You should have received a copy of the GNU General Public License      ##
## along with this program.  If not, see <http://www.gnu.org/licenses/>.  ##
##                                                                        ##
############################################################################


import os
from ConfigParser import SafeConfigParser


class Config(dict):
    """The configuration parser.

    Args:
        filepath -- The path of the configuration file.
    """

    def __init__(self, filepath):
        """The constructor."""
        self._filepath = filepath
        # Default config > Library
        self['library/path'] = ""
        # Load from file
        if os.path.isfile(filepath):
            self._load_from_file()

    def _load_from_file(self):
        """Read the configuration from the given file."""
        # Translation dict (for casting variables)
        cast = {int: int,
                str: str,
                bool: lambda x: bool(int(x)),
                list: lambda x: re.sub(r"\s*,\s*", ",", x).split(",")}
        # Parse the config file
        config_parser = SafeConfigParser()
        config_parser.read(self._filepath)
        for section in config_parser.sections():
            for key, value in config_parser.items(section):
                ckey = '%s/%s' % (section.lower(), key.lower())
                #Update the config
                if ckey in self:
                    self[ckey] = cast[type(self[ckey])](value)

    def __getattr__(self, attr):
        """Sugar...

        Allow you to use the "Config().section_key" syntax instead of
        "Config()['section/key']".
        """
        key = attr.replace("_", "/")
        if key in self:
            return self[key]
        else:
            raise AttributeError("Config object has no '%s' attribute." % attr)


CONFIG = Config("./bookkun.cfg")  # FIXME: filepath
